@@ -2,7 +2,7 @@
 # Copyright 1999-2003 Gentoo Technologies, Inc.
 # Distributed under the terms of the GNU General Public License v2
 # Author:  Martin Schlemmer <azarah@gentoo.org>
-# $Header: /home/cvsroot/gentoo-src/rc-scripts/sbin/depscan.sh,v 1.24 2004/02/26 17:29:43 azarah Exp $
+# $Header: /home/cvsroot/gentoo-src/rc-scripts/sbin/depscan.sh,v 1.25 2004/04/13 00:20:03 agriffis Exp $
 
 source /etc/init.d/functions.sh
 
@@ -16,9 +16,9 @@ then
 	fi
 fi
 
-for x in ${svcdir} softscripts snapshot options started
+for x in softscripts snapshot options started
 do
-	if [ ! -d "${x}" ]
+	if [ ! -d "${svcdir}/${x}" ]
 	then
 		if ! mkdir -p -m 0755 "${svcdir}/${x}" 2>/dev/null
 		then
