@@ -2,7 +2,7 @@
 # Copyright 1999-2003 Gentoo Technologies, Inc.
 # Distributed under the terms of the GNU General Public License v2
 # Author:  Martin Schlemmer <azarah@gentoo.org>
-# $Header: /home/cvsroot/gentoo-src/rc-scripts/sbin/env-update.sh,v 1.1 2003/05/12 21:23:05 azarah Exp $
+# $Header: /home/cvsroot/gentoo-src/rc-scripts/sbin/env-update.sh,v 1.2 2003/12/29 01:00:27 azarah Exp $
 
 
 source /etc/init.d/functions.sh
@@ -22,12 +22,14 @@ note:
 "
 	exit 1
 }
-		
+
+export SVCDIR="${svcdir}"
+
 if [ "$#" -ne 0 ]
 then
 	usage
 else
-	/bin/gawk -v SVCDIR="${svcdir}" \
+	/bin/gawk \
 		-f /lib/rcscripts/awk/functions.awk \
 		-f /lib/rcscripts/awk/genenviron.awk
 fi
