@@ -1,7 +1,6 @@
-# Copyright 1999-2003 Gentoo Technologies, Inc.
+# Copyright 1999-2004 Gentoo Technologies, Inc.
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /home/cvsroot/gentoo-src/rc-scripts/sbin/functions.sh,v 1.49 2004/02/15 20:01:09 azarah Exp $
-
+# $Header: /home/cvsroot/gentoo-src/rc-scripts/sbin/functions.sh,v 1.51 2004/04/24 02:07:00 agriffis Exp $
 
 RC_GOT_FUNCTIONS="yes"
 
@@ -441,7 +440,11 @@ eend() {
 	else
 		retval="$1"
 		
-		rc_splash "stop" &>/dev/null &
+		if [ -c /dev/null ] ; then
+			rc_splash "stop" &>/dev/null &
+		else
+			rc_splash "stop" &
+		fi
 		
 		if [ "$#" -ge 2 ]
 		then
