@@ -1,7 +1,7 @@
 /*
  * Copyright 1999-2003 Gentoo Technologies, Inc.
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /home/cvsroot/gentoo-src/rc-scripts/src/runscript.c,v 1.2 2003/05/12 21:23:05 azarah Exp $
+ * $Header: /home/cvsroot/gentoo-src/rc-scripts/src/runscript.c,v 1.4 2004/03/03 19:55:07 vapier Exp $
  */
 
 
@@ -10,9 +10,13 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <wait.h>
+#include <dlfcn.h>
+
+static void (*selinux_run_init) (void);
 
 int main(int argc, char **argv) {
 	char *myargs[32];
+	void *lib_handle;
 	int new = 1;
 	myargs[0] = "runscript";
 	
@@ -28,6 +32,13 @@ int main(int argc, char **argv) {
 		execv("/lib/rcscripts/sh/rc-help.sh",myargs);
 		exit(1);
 	}
+	
+	lib_handle = dlopen("/lib/rcscripts/runscript_selinux.so", RTLD_LAZY | RTLD_GLOBAL);
+	if( lib_handle != NULL ) {
+		selinux_run_init = dlsym(lib_handle, "selinux_runscript");
+		selinux_run_init();
+	}
+
 	if (execv("/sbin/runscript.sh",myargs) < 0)
 		exit(1);
 
